@@ -1,4 +1,6 @@
-/* xjadeo - jack video monitor
+/* xjadeo - remote control abstraction and command parser
+ *
+ * (C) 2006-2014 Robin Gareus <robin@gareus.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,29 +14,26 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
- *
- * (c) 2006 
- *  Robin Gareus <robin@gareus.org>
- *  Luis Garrido <luisgarrido@users.sourceforge.net>
- *
- *
+ * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
+/**
  *
  * XAPI return values:
- *  1xx: command succeeded 
- *  2xx: query variable succeeded: 
+ *  1xx: command succeeded
+ *  2xx: query variable succeeded:
  *  3xx: async messages (initiated by xjadeo)
  *  4xx: error
  *  8xx: info message (eg. help)
  *
- * more detailed: 
+ * more detailed:
  *  100: <text>
- *  101: var=<int> // long int 
+ *  101: var=<int>
  *  124: vmode=<int> : <string> (list of avail video modes)
  *
  *  201: var=<int>  // long int
  *  202: var=<double>
- *  210: var=<int>x<int> 
+ *  210: var=<int>x<int>
  *  220: var=<string>
  *  228: var=<smpte-string>
  *
@@ -50,16 +49,8 @@
  */
 #include "xjadeo.h"
 
-#ifdef OLD_FFMPEG
-#include <avcodec.h> // needed for PIX_FMT 
-#include <avformat.h>
-#else
-#include <libavcodec/avcodec.h> // needed for PIX_FMT 
+#include <libavcodec/avcodec.h> // needed for PIX_FMT
 #include <libavformat/avformat.h>
-#endif
-
-#include <jack/jack.h>
-#include <jack/transport.h>
 
 #include <time.h>
 #include <getopt.h>
@@ -84,22 +75,22 @@ extern uint8_t           *buffer;
 extern int               render_fmt;
 
 /* Video File Info */
-extern double 	duration;
-extern double 	framerate;
-extern long	frames;
+extern double duration;
+extern double framerate;
+extern int64_t frames;
 
 extern AVFormatContext   *pFormatCtx;
 extern int               videoStream;
 
-extern int 	force_redraw; // tell the main event loop that some cfg has changed
+extern int force_redraw; // tell the main event loop that some cfg has changed
 
 /* Option flags and variables */
 extern char *current_file;
-extern long	ts_offset;
-extern char    *smpte_offset;
-extern long	userFrame;
-extern long	dispFrame;
-extern int want_quiet;
+extern int64_t ts_offset;
+extern char *smpte_offset;
+extern int64_t userFrame;
+extern int64_t dispFrame;
+extern int  want_quiet;
 extern int want_verbose;
 extern int want_letterbox;
 extern int remote_en;
@@ -114,34 +105,26 @@ extern char midiid[32];
 #endif
 
 #ifdef TIMEMAP
-extern long   timeoffset;
-extern double timescale;
-extern int    wraparound;
+extern int64_t timeoffset;
+extern double  timescale;
+extern int     wraparound;
 #endif
 
-extern double 		delay;
-extern double 		filefps;
-extern int		videomode;
-extern int 		seekflags;
+extern double delay;
+extern int    videomode;
 extern int    interaction_override;
 
 // On screen display
-extern char OSD_fontfile[1024]; 
+extern char OSD_fontfile[1024];
 extern char OSD_text[128];
-extern int OSD_mode;
+extern int  OSD_mode;
 
 extern int OSD_fx, OSD_tx, OSD_sx, OSD_fy, OSD_sy, OSD_ty;
 
-extern jack_client_t *jack_client;
-extern char jackid[16];
-
-
-// TODO: someday we can implement 'mkfifo' or open / pipe
 #define REMOTE_RX (fileno(stdin))
 #define REMOTE_TX (fileno(stdout))
 
 #define REMOTEBUFSIZ 4096
-int remote_read(void);
 void remote_printf(int val, const char *format, ...);
 
 //--------------------------------------------
@@ -155,13 +138,13 @@ void xapi_printversion(void *d) {
 void xapi_open(void *d) {
 	char *fn= (char*)d;
 	//printf("open file: '%s'\n",fn);
-	if ( open_movie(fn)) 
+	if ( open_movie(fn))
 		remote_printf(403, "failed to open file '%s'",fn);
 	else	remote_printf(129, "opened file: '%s'",fn);
 	init_moviebuffer();
 	newsourcebuffer();
+	Xletterbox(Xgetletterbox());
 	force_redraw=1;
-	//display_frame((int64_t)(dispFrame),1); // update screen
 }
 
 void xapi_close(void *d) {
@@ -179,8 +162,8 @@ void xapi_close_window(void *d) {
 }
 
 void xapi_set_videomode(void *d) {
-	int vmode; 
-	if (getvidmode() !=0) {
+	int vmode;
+	if (getvidmode() != VO_AUTO) {
 		remote_printf(413, "cannot change videomode while window is open.");
 		return;
 	}
@@ -191,11 +174,11 @@ void xapi_set_videomode(void *d) {
 		return;
 	}
 	render_fmt = vidoutmode(vmode);
-	remote_printf(100, "setting video mode to %i",getvidmode());
+	remote_printf(100, "setting video mode to %i", getvidmode());
 
 	open_window(); // required here; else VOutout callback fn will fail.
 
-	if (pFrameFMT && current_file) { 
+	if (pFrameFMT && current_file) {
 		// reinit buffer with new format
 		char *fn=strdup(current_file);
 		open_movie(fn);
@@ -209,7 +192,7 @@ void xapi_set_videomode(void *d) {
 }
 
 void xapi_open_window(void *d) {
-	if (getvidmode() !=0) {
+	if (getvidmode() != VO_AUTO) {
 		remote_printf(412, "window already open.");
 		return;
 	}
@@ -225,9 +208,9 @@ void xapi_lvideomodes(void *d) {
 	int i=0;
 	remote_printf(100,"list video modes.");
 	while (vidoutsupported(++i)>=0) {
-		if (vidoutsupported(i)) 
+		if (vidoutsupported(i))
 			remote_printf(124,"vmode=%i : %s",i,vidoutname(i));
-		else  
+		else
 			remote_printf(800,"n/a=%i : %s",i,vidoutname(i));
 	}
 }
@@ -249,19 +232,19 @@ void xapi_sletterbox(void *d) {
 
 void xapi_pwinpos(void *d) {
 	int x,y;
-	Xgetpos(&x,&y); 
+	Xgetpos(&x,&y);
 	remote_printf(210,"windowpos=%ix%i",x,y);
 }
 
 void xapi_pwinsize(void *d) {
 	unsigned int x,y;
-	Xgetsize(&x,&y); 
+	Xgetsize(&x,&y);
 	remote_printf(210,"windowsize=%ux%u",x,y);
 }
 
 void xapi_saspect (void *d) {
 	unsigned int my_Width,my_Height;
-	Xgetsize(&my_Width,&my_Height); 
+	Xgetsize(&my_Width,&my_Height);
 	// resize to match movie aspect ratio
 	// dup code in display_x11.c (!)
 	if( movie_aspect < ((float)my_Width/(float)my_Height) )
@@ -279,7 +262,7 @@ void xapi_swinsize(void *d) {
 
 	h=ffctv_height;
 	w=ffctv_width;
-	
+
 	if ((tmp=strchr(size,'x')) && ++tmp) {
 		w=atoi(size);
 		h=atoi(tmp);
@@ -336,23 +319,21 @@ void xapi_mousepointer(void *d) {
 }
 
 void xapi_poverride(void *d) {
-		remote_printf(201,"override=%i", interaction_override);
+	remote_printf(201,"override=%i", interaction_override);
 }
 
 void xapi_soverride(void *d) {
-  interaction_override=atoi(d);
+	interaction_override=atoi(d);
 	//remote_printf(100,"ok.");
 	xapi_poverride(NULL);
 }
 
-
 void xapi_swinpos(void *d) {
-	int x,y;
 	char *t0= (char*)d;
 	char *t1;
-	x=0;y=0;
-	
+
 	if ((t1=strchr(t0,'x')) && ++t1) {
+		int x,y;
 		x=atoi(t0);
 		y=atoi(t1);
 
@@ -373,9 +354,9 @@ void xapi_quit(void *d) {
 }
 
 void xapi_pfilename(void *d) {
-	if (current_file) 
+	if (current_file)
 		remote_printf(220, "filename=%s", current_file);
-	else 
+	else
 		remote_printf(410, "no open video file");
 }
 
@@ -388,17 +369,17 @@ void xapi_pframerate(void *d) {
 }
 
 void xapi_pframes(void *d) {
-	remote_printf(201, "frames=%ld ", frames);
+	remote_printf(201, "frames=%"PRId64, frames);
 }
 
 void xapi_poffset(void *d) {
-	remote_printf(201,"offset=%li",(long int) ts_offset);
+	remote_printf(201,"offset=%"PRId64, ts_offset);
 }
 
 void xapi_ptimescale(void *d) {
 #ifdef TIMEMAP
 	remote_printf(202,"timescale=%g", timescale);
-	remote_printf(201,"timeoffset=%li",(long int) timeoffset);
+	remote_printf(201,"timeoffset=%"PRId64, timeoffset);
 #else
 	remote_printf(499,"timescale is not available.");
 #endif
@@ -413,28 +394,10 @@ void xapi_ploop(void *d) {
 }
 
 void xapi_pseekmode (void *d) {
-	switch (seekflags) {
-		case SEEK_ANY:
-			remote_printf(201,"seekmode=2 # any"); break;
-		case SEEK_KEY:
-			remote_printf(201,"seekmode=3 # keyframe"); break;
-		default:
-			remote_printf(201,"seekmode=1 # continuous"); break;
-	}
+	remote_printf(899,"seekmode is deprecated.");
 }
 
 void xapi_sseekmode (void *d) {
-	char *mode= (char*)d;
-	if (!strcmp(mode,"key") || atoi(mode)==3)
-		seekflags=SEEK_KEY;
-	else if (!strcmp(mode,"any") || atoi(mode)==2)
-		seekflags=SEEK_ANY;
-	else if (!strcmp(mode,"continuous") || atoi(mode)==1)
-		seekflags=SEEK_CONTINUOUS;
-	else { 
-		remote_printf(422,"invalid argument (1-3 or 'continuous', 'any', 'key')");
-		return;
-	}
 	xapi_pseekmode(NULL);
 }
 
@@ -447,18 +410,17 @@ void xapi_pmheight(void *d) {
 }
 
 void xapi_soffset(void *d) {
-  	//long int new = atol((char*)d);
 	ts_offset = smptestring_to_frame((char*)d);
-	remote_printf(101,"offset=%li",(long int) ts_offset);
+	remote_printf(101,"offset=%"PRId64, ts_offset);
 }
 
 void xapi_stimescale(void *d) {
 #ifdef TIMEMAP
-        char *t1;
-  	timescale = atof((char*)d);
+	char *t1;
+	timescale = atof((char*)d);
 	if ((t1=strchr((char*)d,' ')) && ++t1) {
-  	  timeoffset = atol(t1);
-        }
+		timeoffset = atol(t1);
+	}
 	xapi_ptimescale(NULL);
 	force_redraw=1;
 #else
@@ -476,28 +438,27 @@ void xapi_sloop(void *d) {
 
 void xapi_sreverse(void *d) {
 #ifdef TIMEMAP
-  	timescale *= -1.0; 
-	if (timescale<0) 
+	timescale *= -1.0;
+	if (timescale<0)
 		timeoffset = (-2.0*timescale) *dispFrame; // TODO: check file-offset and ts_offset.
-	else 
+	else
 		timeoffset = 0; // TODO - applt diff dispFrame <> transport src
 #endif
 }
 
 void xapi_pposition(void *d) {
-	remote_printf(201,"position=%li",dispFrame);
+	remote_printf(201,"position=%"PRId64, dispFrame);
 }
 
 void xapi_psmpte(void *d) {
 	char smptestr[13];
-	frame_to_smptestring(smptestr,dispFrame);
-	remote_printf(228,"smpte=%s",smptestr);
+	frame_to_smptestring(smptestr, dispFrame, 0);
+	remote_printf(228,"smpte=%s",smptestr, 0);
 }
 
 void xapi_seek(void *d) {
-  	//long int new = atol((char*)d);
-	userFrame= smptestring_to_frame((char*)d);
-	remote_printf(101,"defaultseek=%li",userFrame);
+	userFrame = smptestring_to_frame((char*)d);
+	remote_printf(101,"defaultseek=%"PRId64, userFrame);
 }
 
 void xapi_pfps(void *d) {
@@ -512,49 +473,31 @@ void xapi_sfps(void *d) {
 	remote_printf(101,"updatefps=%i",(int) rint(1/delay));
 }
 
-void xapi_sframerate(void *d) {
-	char *off= (char*)d;
-	if (!(atof(off)>0)) {
-		remote_printf(423,"invalid argument (range >0)"); 
-		return;
-	}
-        filefps= atof(off);
-	override_fps(filefps);
-	remote_printf(202, "framerate=%g", framerate);
+static void xapi_sframerate(void *d) {
+	remote_printf(899,"file framerate is deprecated.");
 }
 
-void xapi_jack_status(void *d) {
-	if (jack_client) 
-		remote_printf(220,"jackclient=%s",jackid);
-	else 
+static void xapi_jack_status(void *d) {
+	if (jack_connected())
+		remote_printf(220,"jackclient=%s", xj_jack_client_name());
+
+	else
 		remote_printf(100,"not connected to jack server");
 }
 
 void xapi_ltc_status(void *d) {
 	if (ltcjack_connected())
-		remote_printf(220,"jackclient=%s",ltc_jack_client_name());
-	else 
+		remote_printf(220,"jackclient=%s", xj_jack_client_name());
+	else
 		remote_printf(100,"no open LTC JACK source");
 }
 
 void xapi_open_ltc(void *d) {
-#if defined (HAVE_LTCSMPTE) || defined (HAVE_LTC)
-#ifdef HAVE_MIDI
-	midi_close();
-#endif
-	close_jack();
-	open_ltcjack((char *) d); // TODO: autoconnect 
-	if (ltcjack_connected())
-		remote_printf(100,"opened LTC jack port.");
-	else 
-		remote_printf(405,"failed to connect to jack server");
-#else
-	remote_printf(499,"LTC-jack is not available.");
-#endif
+	INT_sync_to_ltc((char *) d, 1);
 }
 
 void xapi_close_ltc(void *d) {
-#if defined (HAVE_LTCSMPTE) || defined (HAVE_LTC)
+#ifdef HAVE_LTC
 	close_ltcjack();
 	remote_printf(100,"closed ltc-jack connection");
 #else
@@ -563,17 +506,7 @@ void xapi_close_ltc(void *d) {
 }
 
 void xapi_open_jack(void *d) {
-#ifdef HAVE_MIDI
-	midi_close();
-#endif
-#if defined (HAVE_LTCSMPTE) || defined (HAVE_LTC)
-	close_ltcjack();
-#endif
-	open_jack();
-	if (jack_client) 
-		remote_printf(100,"connected to jack server.");
-	else 
-		remote_printf(405,"failed to connect to jack server");
+	INT_sync_to_jack(1);
 }
 
 void xapi_close_jack(void *d) {
@@ -591,7 +524,7 @@ void xapi_osd_smpte(void *d) {
 		OSD_mode|=OSD_SMPTE;
 		OSD_sy=y;
 		remote_printf(100,"rendering smpte on position y:%i%%",y);
-	} else 
+	} else
 		remote_printf(422,"invalid argument (range -1..100)");
 	force_redraw=1;
 	//display_frame((int64_t)(dispFrame),1); // update OSD
@@ -606,7 +539,7 @@ void xapi_osd_frame(void *d) {
 		OSD_mode|=OSD_FRAME;
 		OSD_fy=y;
 		remote_printf(100,"rendering frame on position y:%i%%",y);
-	} else 
+	} else
 		remote_printf(422,"invalid argument (range -1..100)");
 	force_redraw=1;
 	//display_frame((int64_t)(dispFrame),1); // update OSD
@@ -666,15 +599,28 @@ void xapi_osd_mode(void *d) {
 	if (m&2) OSD_mode|=OSD_SMPTE; else OSD_mode&=~OSD_SMPTE;
 	if (m&4) OSD_mode|=OSD_TEXT;  else OSD_mode&=~OSD_TEXT;
 	if (m&8) OSD_mode|=OSD_BOX;   else OSD_mode&=~OSD_BOX;
-	remote_printf(100,"set osdmode=%i", (OSD_mode&OSD_FRAME?1:0)|(OSD_mode&OSD_SMPTE?2:0)|(OSD_mode&OSD_TEXT?4:0)|(OSD_mode&OSD_BOX?8:0));
+	if (m&16) OSD_mode|=OSD_VTC;   else OSD_mode&=~OSD_VTC;
+	remote_printf(100,"set osdmode=%i",
+			 (OSD_mode&OSD_FRAME ?1:0)
+			|(OSD_mode&OSD_SMPTE ?2:0)
+			|(OSD_mode&OSD_TEXT  ?4:0)
+			|(OSD_mode&OSD_BOX   ?8:0)
+			|(OSD_mode&OSD_VTC   ?16:0)
+			);
 	force_redraw=1;
 }
 
 void xapi_posd(void *d) {
 #ifdef HAVE_FT
-	remote_printf(201,"osdmode=%i", (OSD_mode&OSD_FRAME?1:0)|(OSD_mode&OSD_SMPTE?2:0)|(OSD_mode&OSD_TEXT?4:0)|(OSD_mode&OSD_BOX?8:0));
-	remote_printf(220,"osdfont=%s", OSD_fontfile); 
-	remote_printf(220,"osdtext=%s", OSD_text); 
+	remote_printf(201,"osdmode=%i",
+			 (OSD_mode&OSD_FRAME ?1:0)
+			|(OSD_mode&OSD_SMPTE ?2:0)
+			|(OSD_mode&OSD_TEXT  ?4:0)
+			|(OSD_mode&OSD_BOX   ?8:0)
+			|(OSD_mode&OSD_VTC   ?16:0)
+			);
+	remote_printf(220,"osdfont=%s", OSD_fontfile);
+	remote_printf(220,"osdtext=%s", OSD_text);
 #else
 	remote_printf(490,"this feature is not compiled");
 #endif
@@ -682,7 +628,7 @@ void xapi_posd(void *d) {
 
 void xapi_psync(void *d) {
 	int ss =0;
-#if defined (HAVE_LTCSMPTE) || defined (HAVE_LTC)
+#ifdef HAVE_LTC
 	if (ltcjack_connected()) ss=3;
 	else
 #endif
@@ -697,7 +643,7 @@ void xapi_psync(void *d) {
 void xapi_osd_pos(void *d) {
 	char *t0= (char*)d;
 	char *t1;
-	
+
 	if ((t1=strchr(t0,' ')) && ++t1) {
 		OSD_tx=atoi(t0);
 		OSD_ty=atoi(t1);
@@ -716,7 +662,7 @@ void xapi_osd_pos(void *d) {
 
 void xapi_midi_status(void *d) {
 #ifdef HAVE_MIDI
-	// FIXME: we shall return "200,midiid=XXX" 
+	// FIXME: we shall return "200,midiid=XXX"
 	// and "100 not connected" ?? as in jack connected
 	// BUT: // midiid can be <int> (portmidi) or string (alsa midi)
 	if (midi_connected())
@@ -731,7 +677,7 @@ void xapi_midi_status(void *d) {
 void xapi_smididriver(void *d) {
 #ifdef HAVE_MIDI
 	char *mp = NULL;
-	if (d && strlen(d)>0) mp=d; 
+	if (d && strlen(d)>0) mp=d;
 	if (midi_choose_driver(mp)>0) {
 		remote_printf(100,"ok.");
 	} else {
@@ -743,9 +689,13 @@ void xapi_smididriver(void *d) {
 }
 
 void xapi_open_midi(void *d) {
+	if (jack_connected()) close_jack();
+#ifdef HAVE_LTC
+	if (ltcjack_connected()) close_ltcjack();
+#endif
 #ifdef HAVE_MIDI
 	char *mp;
-	if (d && strlen(d)>0) mp=d; 
+	if (d && strlen(d)>0) mp=d;
 	else mp="-1"; // midiid ?
 
 	if (midi_connected())
@@ -798,7 +748,7 @@ void xapi_pmidilibrary (void *d) {
 #endif
 }
 
-	
+
 void xapi_pmidiclk(void *d) {
 #ifdef HAVE_MIDI
 	remote_printf(201,"midiclk=%i", midi_clkadj);
@@ -809,14 +759,14 @@ void xapi_pmidiclk(void *d) {
 
 void xapi_smidiclk(void *d) {
 #ifdef HAVE_MIDI
-        midi_clkadj = atoi((char*)d)?1:0;
+	midi_clkadj = atoi((char*)d)?1:0;
 	remote_printf(101,"midiclk=%i", midi_clkadj);
 #else
 	remote_printf(499,"midi not available.");
 #endif
 }
 
-	
+
 void xapi_pmidisync(void *d) {
 #ifdef HAVE_MIDI
 	remote_printf(201,"midisync=%i", midi_clkconvert);
@@ -827,7 +777,7 @@ void xapi_pmidisync(void *d) {
 
 void xapi_smidisync(void *d) {
 #ifdef HAVE_MIDI
-        midi_clkconvert = atoi((char*)d);
+	midi_clkconvert = atoi((char*)d);
 	remote_printf(101,"midisync=%i", midi_clkconvert);
 #else
 	remote_printf(499,"midi not available.");
@@ -887,10 +837,10 @@ void xapi_null(void *d) {
 	remote_printf(402,"command not implemented.");
 }
 
-void api_help(void *d);
+static void api_help(void *d);
 
 //--------------------------------------------
-// cmd interpreter 
+// cmd interpreter
 //--------------------------------------------
 typedef void myfunction (void *);
 
@@ -902,43 +852,38 @@ typedef struct _command {
 	int sticky;  // unused
 } Dcommand;
 
-Dcommand cmd_test[] = {
-	{"load ", "<filename>: replace current video file.", NULL , xapi_open, 0 },
-	{"seek ", "<int>: seek to this frame - if jack and midi are offline", NULL, xapi_seek , 0 },
-	{NULL, NULL, NULL , NULL, 0}
-};
 
-Dcommand cmd_midi[] = {
+static Dcommand cmd_midi[] = {
 /*	{"autoconnect", ": discover and connect to midi time source", NULL, xapi_detect_midi, 0 }, */
 	{"connect ", "<port>: connect to midi time source (-1: discover)", NULL, xapi_open_midi, 0 },
-	{"disconnect", ": unconect from midi device", NULL, xapi_close_midi, 0 },
+	{"disconnect", ": close midi device", NULL, xapi_close_midi, 0 },
 	{"reconnect", ": connect to last specified midi port", NULL, xapi_reopen_midi, 0 },
-	{"status", ": display status of midi connection.", NULL, xapi_midi_status, 0 },
-	{"driver ", "<name>: select midi driver.", NULL, xapi_smididriver, 0 },
+	{"status", ": display status of midi connection", NULL, xapi_midi_status, 0 },
+	{"driver ", "<name>: select midi driver", NULL, xapi_smididriver, 0 },
 	{"driver", ": display the used midi driver", NULL, xapi_pmidilibrary, 0 },
 	{"library", ": alias for 'midi driver' (deprecated)", NULL, xapi_pmidilibrary, 0 },
 	{"sync ", "<int>: set MTC smpte conversion. 0:MTC 2:Video 3:resample", NULL, xapi_smidisync, 0 },
-	{"clk ", "[1|0]: use MTC quarter frames.", NULL, xapi_smidiclk, 0 },
+	{"clk ", "[1|0]: use MTC quarter frames", NULL, xapi_smidiclk, 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_jack[] = {
+static Dcommand cmd_jack[] = {
 	{"connect", ": connect and sync to jack server", NULL, xapi_open_jack , 0 },
 	{"disconnect", ": disconnect from jack server", NULL, xapi_close_jack , 0 },
 	{"status", ": get status of jack connection", NULL, xapi_jack_status , 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_ltc[] = {
-	{"connect", ": connect and sync to jack server", NULL, xapi_open_ltc , 0 },
-	{"disconnect", ": disconnect from jack server", NULL, xapi_close_ltc , 0 },
+static Dcommand cmd_ltc[] = {
+	{"connect", ": connect and sync to LTC server", NULL, xapi_open_ltc , 0 },
+	{"disconnect", ": disconnect from LTC", NULL, xapi_close_ltc , 0 },
 	{"status", ": get status of jack connection", NULL, xapi_ltc_status , 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_osd[] = {
-	{"frame " , "<ypos>: render current framenumber. y=0..100 (<0 disable)", NULL, xapi_osd_frame, 0 },
-	{"smpte " , "<ypos>: render smpte. y=0..100 (<0 disable)", NULL, xapi_osd_smpte, 0 },
+static Dcommand cmd_osd[] = {
+	{"frame " , "<ypos>: render current frame number. y=0..100 (negative integer: disable)", NULL, xapi_osd_frame, 0 },
+	{"smpte " , "<ypos>: render sync timecode. y=0..100 (negative integer: disable)", NULL, xapi_osd_smpte, 0 },
 	{"text " , "<text>: render <text> on screen", NULL, xapi_osd_text, 0 },
 	{"text" , ": display prev. OSD text", NULL, xapi_osd_on, 0 },
 	{"notext" , ": clear text OSD", NULL, xapi_osd_off, 0 },
@@ -948,12 +893,12 @@ Dcommand cmd_osd[] = {
 	{"available" , ": return 100 if freetype OSD is available", NULL, xapi_osd_avail, 0 },
 	{"font " , "<filename>: use this TTF font file", NULL, xapi_osd_font, 0 },
 	{"box" , ": forces a black box around the OSD", NULL, xapi_osd_box, 0 },
-	{"nobox" , ": make OSD backgroung transparent", NULL, xapi_osd_nobox, 0 },
-	{"mode" , ":<int>: restore OSD as returned by 'get osdcfg'", NULL, xapi_osd_mode, 0 },
+	{"nobox" , ": transparent OSD background", NULL, xapi_osd_nobox, 0 },
+	{"mode" , "<int>: restore OSD as returned by 'get osdcfg'", NULL, xapi_osd_mode, 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_get[] = {
+static Dcommand cmd_get[] = {
 	{"position", ": return current frame position", NULL, xapi_pposition , 0 },
 	{"smpte", ": return current frame position", NULL, xapi_psmpte , 0 },
 	{"fps", ": display current update frequency", NULL, xapi_pfps , 0 },
@@ -968,11 +913,11 @@ Dcommand cmd_get[] = {
 	{"width", ": query width of video source buffer", NULL, xapi_pmwidth , 0 },
 	{"height", ": query width of video source buffer", NULL, xapi_pmheight , 0 },
 
-	{"seekmode", ": display how video frames are searched ", NULL, xapi_pseekmode, 0 },
+	{"seekmode", ": deprecated - no return value", NULL, xapi_pseekmode, 0 },
 	{"windowsize" , ": show current window size", NULL, xapi_pwinsize, 0 },
 	{"windowpos" , ": show current window position", NULL, xapi_pwinpos, 0 },
 	{"videomode" , ": display current video mode", NULL, xapi_pvideomode, 0 },
-	{"midisync", ": display midi smpte conversion mode", NULL, xapi_pmidisync, 0 },
+	{"midisync", ": display midi SMPTE conversion mode", NULL, xapi_pmidisync, 0 },
 	{"midiclk", ": MTC quarter frame precision", NULL, xapi_pmidiclk, 0 },
 	{"osdcfg", ": display status on screen display", NULL, xapi_posd, 0 },
 	{"syncsource", ": display currently used sync source", NULL, xapi_psync, 0 },
@@ -984,71 +929,71 @@ Dcommand cmd_get[] = {
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_notify[] = {
+static Dcommand cmd_notify[] = {
 	{"disable" , ": disable async messages", NULL, xapi_bidir_alloff, 0 },
-	{"frame" , ": enable async frame-update messages", NULL, xapi_bidir_frame, 0 },
-	{"keyboard" , ": enable async keypress messages", NULL, xapi_bidir_keyboard, 0 },
+	{"off" , ": alias for 'disable'", NULL, xapi_bidir_alloff, 0 },
+	{"frame" , ": subscribe to async frame-update messages", NULL, xapi_bidir_frame, 0 },
+	{"keyboard" , ": subscribe to async key-press notifications", NULL, xapi_bidir_keyboard, 0 },
 	{"loop" , ": enable continuous frame position messages", NULL, xapi_bidir_loop, 0 },
+	{"settings" , ": receive a dump of current settings when xjadeo shuts down", NULL, xapi_bidir_settings, 0 },
+	{"noframe" , ": stop frame-update message subscription", NULL, xapi_bidir_noframe, 0 },
+	{"noloop" , ": disable continuous frame position messages", NULL, xapi_bidir_noloop, 0 },
+	{"nokeyboard" , ": disable async key-press notification messages", NULL, xapi_bidir_nokeyboard, 0 },
 	{"nosettings" , ": disable async settings dump on shutdown", NULL, xapi_bidir_nosettings, 0 },
-	{"nokeyboard" , ": disable async keypress messages", NULL, xapi_bidir_nokeyboard, 0 },
-	{"noframe" , ": enable async frame-update messages", NULL, xapi_bidir_noframe, 0 },
-	{"noloop" , ": enable continuous frame position messages", NULL, xapi_bidir_noloop, 0 },
-	{"off" , ": disable all async messages", NULL, xapi_bidir_alloff, 0 },
-	{"settings" , ": enable async settings dump on shutdown", NULL, xapi_bidir_settings, 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_window[] = {
+static Dcommand cmd_window[] = {
 	{"close", ": close window", NULL, xapi_close_window, 0 },
 	{"open", ": open window", NULL, xapi_open_window, 0 },
-	{"mode " , "<int>: changes video mode and opens window", NULL, xapi_set_videomode, 0 },
-	{"resize " , "<int>|<int>x<int>: resize window (percent of movie or abs)", NULL, xapi_swinsize, 0 },
+	{"mode " , "<int>: change video mode and open window", NULL, xapi_set_videomode, 0 },
+	{"resize " , "<int>|<int>x<int>: resize window (percent of movie or absolute size)", NULL, xapi_swinsize, 0 },
 	{"size " , "<int>|<int>x<int>: alias for resize", NULL, xapi_swinsize, 0 },
-	{"position " , "<int>x<int>: move window to absolute position", NULL, xapi_swinpos, 0 },
+	{"position " , "<int>x<int>: move window to given position (top-left coordinates)", NULL, xapi_swinpos, 0 },
 	{"pos " , "<int>x<int>: alias for 'window position'", NULL, xapi_swinpos, 0 },
 	{"xy " , "<int>x<int>: alias for 'window position'", NULL, xapi_swinpos, 0 },
-	{"fullscreen " , "[on|off|toggle]: en/disable fullscreen (only XV/x11/OSX)", NULL, xapi_fullscreen, 0 },
-	{"zoom " , "[on|off|toggle]: alias for 'window fullscreen'", NULL, xapi_fullscreen, 0 },
-	{"letterbox " , "[on|off|toggle]: don't break aspect ratio (only XV/x11-imlib2)", NULL, xapi_sletterbox, 0 },
-	{"mouse " , "[on|off|toggle]: en/disable mouse cursor display (only XV/x11)", NULL, xapi_mousepointer, 0 },
-	{"ontop " , "[on|off|toggle]: en/disable 'on top' (only XV/x11)", NULL, xapi_ontop, 0 },
-	{"fixaspect" , ": scale window to match aspect ration (same as mouse btn 3)", NULL, xapi_saspect, 0 },
+	{"fullscreen " , "[on|off|toggle]: en/disable full screen", NULL, xapi_fullscreen, 0 },
+	{"zoom " , "[on|off|toggle]: alias for 'window full screen'", NULL, xapi_fullscreen, 0 },
+	{"letterbox " , "[on|off|toggle]: don't break aspect ratio", NULL, xapi_sletterbox, 0 },
+	{"mouse " , "[on|off|toggle]: en/disable mouse cursor display", NULL, xapi_mousepointer, 0 },
+	{"ontop " , "[on|off|toggle]: en/disable 'on top'", NULL, xapi_ontop, 0 },
+	{"fixaspect" , ": scale window to match aspect ration", NULL, xapi_saspect, 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_set[] = {
-	{"fps ", "<float>: set current update frequency", NULL, xapi_sfps , 0 },
-	{"offset ", "<int>: set current frame offset", NULL, xapi_soffset , 0 },
-	{"timescale ", "<float> <int>: set timescale and offset", NULL, xapi_stimescale , 0 },
-	{"loop ", "<int>: 0: normal, 1:wrap around", NULL, xapi_sloop , 0 },
-	{"seekmode ", "<1-3>: seek continuous, to any or to keyframes only", NULL, xapi_sseekmode, 0 },
-	{"framerate ", "<float>: override frame rate of video file", NULL, xapi_sframerate , 0 },
-	{"override ", "<int>: disable window events", NULL, xapi_soverride , 0 },
+static Dcommand cmd_set[] = {
+	{"offset ", "<int>: set timecode offset in frames", NULL, xapi_soffset , 0 },
+	{"fps ", "<float>: set screen update frequency", NULL, xapi_sfps , 0 },
+	{"framerate ", ": deprecated - no operation", NULL, xapi_sframerate , 0 },
+	{"override ", "<int>: disable user-interaction (bitmask)", NULL, xapi_soverride , 0 },
+	{"seekmode ", ": deprecated - no operation", NULL, xapi_sseekmode, 0 },
+	{"timescale ", "<float> <int>: set timescale and offset (*)", NULL, xapi_stimescale , 0 },
+	{"loop ", "<int>: 0: normal, 1:wrap around (*)", NULL, xapi_sloop , 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
-Dcommand cmd_root[] = {
-	// note: keep 'seek' on top of the list - if an external app wants seek a lot, xjadeo will 
+static Dcommand cmd_root[] = {
+	// note: keep 'seek' on top of the list - if an external app wants seek a lot, xjadeo will
 	// not spend time comparing command strings - OTOH I/O takes much longer than this anyway :X
 	{"seek ", "<int>: seek to this frame - if jack and midi are offline", NULL, xapi_seek , 0 },
-	{"load ", "<filename>: replace current video file.", NULL , xapi_open, 0 },
-	{"unload", ": close video file.", NULL , xapi_close, 0 },
+	{"load ", "<filename>: replace current video file", NULL , xapi_open, 0 },
+	{"unload", ": close video file", NULL , xapi_close, 0 },
 
 	{"window", " .. : monitor window functions", cmd_window, NULL, 0 },
-	{"notify", " .. : async remote info messages", cmd_notify, NULL, 0 },
-	{"get", " .. : query xjadeo variables or state", cmd_get, NULL, 0 },
-	{"set", " .. : set xjadeo variables", cmd_set, NULL, 0 },
 	{"osd", " .. : on screen display commands", cmd_osd, NULL, 0 },
 	{"jack", " .. : jack sync commands", cmd_jack, NULL, 0 },
 	{"midi", " .. : midi sync commands", cmd_midi, NULL, 0 },
 	{"ltc", " ..  : LTC sync commands", cmd_ltc, NULL, 0 },
-	{"reverse", ": set timescale to reverse playback from now", NULL , xapi_sreverse, 0 },
+	{"notify", " .. : async remote info messages", cmd_notify, NULL, 0 },
+	{"get", " .. : query xjadeo variables or state", cmd_get, NULL, 0 },
+	{"set", " .. : set xjadeo variables", cmd_set, NULL, 0 },
+	{"reverse", ": set timescale to reverse playback (*)", NULL , xapi_sreverse, 0 },
 
-	{"list videomodes" , ": displays a list of possible video modes", NULL, xapi_lvideomodes, 0 },
+	{"list videomodes" , ": displays a list of available video modes", NULL, xapi_lvideomodes, 0 },
 	{"ping", ": claim a pong", NULL , xapi_ping, 0 },
-	{"exit", ": [close MQ-session]", NULL , xapi_exit, 0 },
-	{"help", ": show a quick help", NULL , api_help, 0 },
-	{"quit", ": terminate  xjadeo", NULL , xapi_quit, 0 },
+	{"help", ": generate this remote-control API documentation", NULL , api_help, 0 },
+	{"exit", ": close remote-session (keep xjadeo running, with MQ and IPC)", NULL , xapi_exit, 0 },
+	{"quit", ": terminate xjadeo", NULL , xapi_quit, 0 },
 	{NULL, NULL, NULL , NULL, 0},
 };
 
@@ -1057,7 +1002,7 @@ Dcommand cmd_root[] = {
 //  - query OSD status (qjadeo - reconnect)
 //  - query midi settings  (xapi_midi_status)
 
-void api_help_recursive(Dcommand *r, const char *prefix) {
+static void api_help_recursive(Dcommand *r, const char *prefix) {
 	int i=0;
 	while (r[i].name) {
 		if (r[i].children) {
@@ -1074,50 +1019,37 @@ void api_help_recursive(Dcommand *r, const char *prefix) {
 	}
 }
 
-void api_help(void *d) {
+static void api_help(void *d) {
 	remote_printf(100, "print help");
 	remote_printf(800, "+ xjadeo remote control commands:");
 	api_help_recursive(cmd_root,"");
 }
 
-void exec_remote_cmd_recursive (Dcommand *leave, char *cmd) {
+static void exec_remote_cmd_recursive (Dcommand *leave, char *cmd) {
 	int i=0;
 	while (*cmd==' ') ++cmd;
 //	fprintf(stderr,"DEBUG: %s\n",cmd);
 
 	while (leave[i].name) {
-		if (strncmp(cmd,leave[i].name,strlen(leave[i].name))==0) break; 
+		if (strncmp(cmd,leave[i].name,strlen(leave[i].name))==0) break;
 		i++;
 	}
 	if (!leave[i].name) {
 		remote_printf(400,"unknown command.");
 		return; // no cmd found
 	}
-	if (leave[i].children) 
+	if (leave[i].children)
 		exec_remote_cmd_recursive(leave[i].children,cmd+strlen(leave[i].name));
 	else if (leave[i].func) {
 		char *arg= cmd+strlen(leave[i].name);
 		strtok(arg, "\r\n");
 		leave[i].func(arg);
-	} else 
+	} else
 		remote_printf(401,"command not implemented.");
 }
 
-#ifdef HAVE_LASH
-# if !(defined HAVE_MQ || defined HAVE_IPCMSG)
-#warning 
-#warning 
-#warning LASH support - but no POSIX message queues!
-#warning 
-#warning This xjadeo will not be able to (re)connect 
-#warning to a GUI when launched by lashd!
-#warning 
-#warning 
-# endif
-#endif
-
 //--------------------------------------------
-// remote control - STDIO 
+// remote control - STDIO
 //--------------------------------------------
 
 
@@ -1139,8 +1071,8 @@ int remote_read_io(void) {
 
 	while ((end = strchr(start, '\n'))) {
 		*(end) = '\0';
-		//if (strlen(start) > 0) 
-			exec_remote_cmd_recursive(cmd_root,start);
+		//if (strlen(start) > 0)
+		exec_remote_cmd_recursive(cmd_root,start);
 		inbuf->offset-=((++end)-start);
 		if (inbuf->offset) memmove(inbuf->buf,end,inbuf->offset);
 	}
@@ -1152,27 +1084,28 @@ void exec_remote_cmd (char *cmd) {
 	exec_remote_cmd_recursive(cmd_root, cmd);
 }
 
-#ifdef HAVE_WINDOWS
+#ifdef PLATFORM_WINDOWS
 int remote_read_h(void) {
 	int rv=-1;
 	char buf[BUFSIZ];
 	DWORD bytesAvail = 0;
 	HANDLE h = GetStdHandle(STD_INPUT_HANDLE);
 	do {
+		bytesAvail = 0;
 		PeekNamedPipe(h, 0, 0, 0, &bytesAvail, 0);
 		DWORD sr=0;
-		if (bytesAvail > 0 && ReadFile(h, buf, BUFSIZ, &sr, NULL) && sr>0) {
+		if (bytesAvail > 0 && ReadFile(h, buf, BUFSIZ, &sr, NULL) && sr > 0) {
 			buf[sr]=0;
 			char *start, *end;
 			start = buf;
 			while (*start && (end = strchr(start, '\n'))) {
 				*(end) = '\0';
 				strtok(start, "\r");
-				//if (strlen(start) > 0) 
-					exec_remote_cmd_recursive(cmd_root,start);
+				//if (strlen(start) > 0)
+				exec_remote_cmd_recursive(cmd_root,start);
 				start=end+1;
 			}
-		  rv=0;
+			rv=0;
 		}
 	} while (bytesAvail>0);
 	return rv;
@@ -1188,7 +1121,7 @@ int remote_read_h(void) {
 # define LOGLEN 1023
 #endif
 
-void remote_printf_io(int rv, const char *format, ...) {
+static void remote_printf_io(int rv, const char *format, ...) {
 	va_list arglist;
 	char text[LOGLEN];
 	char msg[LOGLEN];
@@ -1199,7 +1132,7 @@ void remote_printf_io(int rv, const char *format, ...) {
 
 	text[LOGLEN -1] =0; // just to be safe :)
 	snprintf(msg, LOGLEN, "@%i %s\n",rv,text);
-	msg[LOGLEN -1] =0; 
+	msg[LOGLEN -1] =0;
 	(void) write(REMOTE_TX,msg,strlen(msg));
 }
 
@@ -1218,7 +1151,6 @@ int remote_fd_set(fd_set *fd) {
 	return( REMOTE_RX+1);
 }
 
-
 //--------------------------------------------
 // POSIX message queeue
 //--------------------------------------------
@@ -1230,9 +1162,8 @@ void mymq_reply(int rv, char *str);
 void mymq_close(void);
 int mymq_init(char *id);
 
-
 /* MQ replacement for remote_printf() */
-void remote_printf_mq(int rv, const char *format, ...) {
+static void remote_printf_mq(int rv, const char *format, ...) {
 	va_list arglist;
 	char text[MQLEN];
 
@@ -1250,7 +1181,7 @@ int remote_read_mq(void) {
 	char *t;
 	int rv = -1;
 
-	while ((rx=mymq_read(data)) > 0 ) { 
+	while ((rx=mymq_read(data)) > 0 ) {
 		if ((t =  strchr(data, '\n'))) *t='\0';
 		//if (strlen(data) < 1) continue;
 		exec_remote_cmd_recursive(cmd_root,data);
@@ -1291,8 +1222,8 @@ int msqrx = 0;
 int myipc_reply(int id, char *msg){
 	struct msgbuf1 txbuf;
 	txbuf.mtype=1;
-  snprintf(txbuf.mtext, BUFSIZ, "@%i %s\n", id, msg); // XXX
-  if (msgsnd(msqtx, (const void*) &txbuf, strlen(txbuf.mtext), IPC_NOWAIT) == -1) {
+	snprintf(txbuf.mtext, BUFSIZ, "@%i %s\n", id, msg); // XXX
+	if (msgsnd(msqtx, (const void*) &txbuf, strlen(txbuf.mtext), IPC_NOWAIT) == -1) {
 		fprintf(stderr, "msgsnd failed., Error = %d: %s\n", errno, strerror(errno));
 		return -1;
 	}
@@ -1319,8 +1250,8 @@ int remote_read_ipc () {
 	s=rxbuf.mtext; // TODO remember end of long messages..
 	while (s && *s && (t = strchr(s, '\n'))) {
 		*t='\0';
-	  if (strlen(s) < 1) continue;
-	  exec_remote_cmd_recursive(cmd_root,s);
+		if (strlen(s) < 1) continue;
+		exec_remote_cmd_recursive(cmd_root,s);
 		s=t+1;
 	}
 	remote_read_ipc(); // read all queued messages..
@@ -1329,8 +1260,8 @@ int remote_read_ipc () {
 
 int open_ipcmsg_ctrl (const char *queuename) {
 
-  key_t key_tx = ftok (queuename, 'a');
-  key_t key_rx = ftok (queuename, 'b');
+	key_t key_tx = ftok (queuename, 'a');
+	key_t key_rx = ftok (queuename, 'b');
 
 	msqrx = msgget(key_rx, IPC_CREAT| S_IRUSR | S_IWUSR);
 	msqtx = msgget(key_tx, IPC_CREAT| S_IRUSR | S_IWUSR);
@@ -1351,28 +1282,28 @@ void close_ipcmsg_ctrl () {
 #endif
 
 //--------------------------------------------
-// REMOTE + MQ wrapper 
+// REMOTE + MQ wrapper
 //--------------------------------------------
 
-void remote_printf_argslist(int rv, const char *format, va_list arglist) {
+static void remote_printf_argslist(int rv, const char *format, va_list arglist) {
 	char text[LOGLEN];
 	char msg[LOGLEN];
 
 	vsnprintf(text, MQLEN, format, arglist);
 
-	text[LOGLEN -1] =0; 
+	text[LOGLEN -1] =0;
 #ifdef HAVE_MQ
-		/* remote_printf_mq(...) */
+	/* remote_printf_mq(...) */
 	mymq_reply(rv,text);
 #elif HAVE_IPCMSG
-		/* remote_printf_ipc(...) */
+	/* remote_printf_ipc(...) */
 	if (ipc_queue) myipc_reply(rv,text);
 #endif
 
-		/* remote_printf_io(...) */
+	/* remote_printf_io(...) */
 	if (remote_en) {
 		snprintf(msg, LOGLEN, "@%i %s\n",rv,text);
-		msg[LOGLEN -1] =0; 
+		msg[LOGLEN -1] =0;
 		(void) write(REMOTE_TX,msg,strlen(msg));
 	}
 }
